#!/usr/bin/tclsh
## factorup.tcl
## by dudliohara@yahoo.com  2003.01.30  version 0.9

set username dudlio
set password ilikesieving
set delfile  1650.del     ;# Where to find factors, ie. SoB.del or Sobstatus.dat

set batchsize 10          ;# How many factors to upload with each HTTP POST

# end user config



package require http 2.3  ;# ie. TCL 8.3



# called every $batchsize
proc _submit { factors cookie l_verified_new} {

    set token [http::geturl http://www.seventeenorbust.com/sieve/sievePost.mhtml -query [http::formatQuery factors $factors] -headers [list Cookie $cookie]]

    upvar #0 $token state

    regexp {<h2>Verification Results</h2>(.*)<h2>} $state(body) match results

    regexp {<tt>([0-9]+) of [0-9]+} $results match num_verified

    regexp {<br>([0-9]+) of the results were new} $results match num_new

    return [list \
            [expr $num_verified + [lindex $l_verified_new 0]] \
            [expr $num_new + [lindex $l_verified_new 1]]]

}

# get cookie
http::config -useragent "Mozilla"

puts "Logging in... $username"
if {[catch {set token [http::geturl http://www.seventeenorbust.com/login/?[http::formatQuery -login true -username $username -password $password]]} catch_error]} { puts "Unable to connect"; exit }

upvar #0 $token state

array set meta $state(meta)

set mycookie ""
catch {set mycookie [lindex [split $meta(Set-Cookie) ;] 0]}


# test the cookie against account page
puts "Testing login... $mycookie"
set token [http::geturl http://www.seventeenorbust.com/account/ -headers [list Cookie $mycookie]]

upvar #0 $token state

if [regexp {Account Preferences} $state(body)] { puts "OK" } else { puts "Login failed"; exit } 


# look for factors
puts "Opening factor file... $delfile"
if [catch {set fp [open $delfile r]} catch_error] { puts "Open failed"; exit }
puts "OK"

puts -nonewline "Submitting"
flush stdout

# ok let's go
set factors ""

set num_lines -1  ;# eof counts as a line otherwise
set num_factors 0
set num_factors_local 0
set num_errors 0

set total_verified 0
set total_new 0

set l_verified_new [list 0 0]

while { ![eof $fp]} {

    incr num_lines

    gets $fp oneline
    
    # strip factors out of the $sobdel file
    if [regexp {[0-9]+ \| [0-9]+\*2\^[0-9]+\+1} $oneline match] { 
        incr num_factors
        incr num_factors_local
        set factors $factors$match\n 
    }

    # submit a batch
    if [expr $num_factors_local == $batchsize] {
        
        if {[catch {set l_verified_new [_submit $factors $mycookie $l_verified_new ]} ]} {
            puts -nonewline "E"
            incr num_errors
        } else {
            puts -nonewline "."    
        }

        flush stdout
        set num_factors_local 0
        set factors ""
    }
}

# submit leftover batch
if {$factors != "" } {
    
    if {[catch {set l_verified_new [_submit $factors $mycookie $l_verified_new]} ]} {
        puts -nonewline "E"
        incr num_errors
    } else {
        puts -nonewline "."    
    }
    
    flush stdout
}

close $fp

# how was it...
puts "\n$num_lines lines"
puts "$num_factors factors"

puts "[lindex $l_verified_new 0] Verified.  [lindex $l_verified_new 1] New."

if {[expr $num_errors > 0]} {
    puts "There were $num_errors batch errors."
    puts "  Up to [expr $num_errors * $batchsize] factors are unaccounted for.  Try running the script again."
}

puts "END"

exit

# END